#!/usr/bin/env bash
set -eo pipefail

RED="\e[31m"
GREEN="\e[32m"
YELLOW="\e[33m"
END="\e[0m\n"

if ! [ -x "$(command -v psql)" ]; then
  printf "${RED}Error: psql is not installed.${END}"
  exit 1
fi
if ! [ -x "$(command -v sqlx)" ]; then
  printf "${RED}Error: sqlx is not installed.${END}"
  exit 1
fi

DB_USER=${POSTGRES_USER:=postgres}
DB_PASSWORD="${POSTGRES_PASSWORD:=password}"
DB_NAME="${POSTGRES_DB:=newsletter}"
DB_PORT="${POSTGRES_PORT:=5432}"
DB_HOST="${POSTGRES_HOST:=localhost}"

if [[ -z "${SKIP_DOCKER}" ]]
then
  printf "${YELLOW}Start postgres container...${END}"
  docker run \
      -e POSTGRES_USER=${DB_USER} \
      -e POSTGRES_PASSWORD=${DB_PASSWORD} \
      -e POSTGRES_DB=${DB_NAME} \
      -p "${DB_PORT}":5432 \
      -d postgres \
      postgres -N 1000
  printf "${GREEN}Postgres container started!${END}"
fi

export PGPASSWORD="${DB_PASSWORD}"
until psql -h "${DB_HOST}" -U "${DB_USER}" -p "${DB_PORT}" -d "postgres" -c '\q' > /dev/null 2>&1; do
  	printf "${YELLOW}Postgres is still unavailable - sleeping...${END}"
	sleep 1
done
printf "${GREEN}Postgres is up and running on port ${DB_PORT}!${END}"

DATABASE_URL=postgres://${DB_USER}:${DB_PASSWORD}@${DB_HOST}:${DB_PORT}/${DB_NAME}

printf "${YELLOW}Start migration...${END}"
export DATABASE_URL
sqlx database create
sqlx migrate run

printf "${YELLOW}Writing .env...${END}"
printf "DATABASE_URL=${DATABASE_URL}\n" > .env
printf "DB_USER=${DB_USER}\n" >> .env
printf "DB_PASSWORD=${DB_PASSWORD}\n" >> .env
printf "DB_PORT=${DB_PORT}\n" >> .env
printf "DB_HOST=${DB_HOST}\n" >> .env
printf "DB_NAME=${DB_NAME}\n" >> .env
printf "EMAIL_USER=${EMAIL_USER}\n" >> .env
printf "EMAIL_SENDER=${EMAIL_SENDER}\n" >> .env
printf "EMAIL_HOST=${EMAIL_HOST}\n" >> .env
printf "EMAIL_PASSWORD=${EMAIL_PASSWORD}\n" >> .env

printf "${GREEN}Postgres has been migrated, ready to go!${END}"
