//! src/configuration.rs
use dotenv::dotenv;
use lettre::transport::smtp::authentication::Credentials;
use lettre::AsyncSmtpTransport;
use lettre::Tokio1Executor;
use secrecy::ExposeSecret;
use secrecy::Secret;
use serde::Deserialize;
use sqlx::{Connection, Executor, PgConnection, PgPool};
use uuid::Uuid;

use crate::email_client::EmailClient;

#[derive(Deserialize)]
pub struct DatabaseConfig {
    pub user: String,
    pub password: Secret<String>,
    pub port: u16,
    pub host: String,
    pub name: String,
}

pub fn parse_db_config() -> DatabaseConfig {
    dotenv().ok();
    envy::prefixed("DB_")
        .from_env::<DatabaseConfig>()
        .expect("Missing database config environment variables")
}

impl DatabaseConfig {
    pub fn server_connection_string(&self) -> Secret<String> {
        Secret::new(format!(
            "postgres://{}:{}@{}:{}",
            self.user,
            self.password.expose_secret(),
            self.host,
            self.port
        ))
    }

    pub fn db_connection_string(&self) -> Secret<String> {
        Secret::new(format!(
            "{}/{}",
            self.server_connection_string().expose_secret(),
            self.name
        ))
    }

    pub async fn connection_pool(&self) -> PgPool {
        PgPool::connect(self.db_connection_string().expose_secret())
            .await
            .expect("Failed to connect to Postgres.")
    }

    pub async fn test_connection_pool(&mut self) -> PgPool {
        let test_db_name = Uuid::new_v4().to_string();
        let server_connection_string = self.server_connection_string();
        let create_db_query = format!(r#"CREATE DATABASE "{}";"#, test_db_name);

        PgConnection::connect(server_connection_string.expose_secret())
            .await
            .expect("Failed to connect to Postgres")
            .execute(create_db_query.as_str())
            .await
            .expect("Failed to create database.");

        let test_db_connection_string = format!(
            "{}/{}",
            server_connection_string.expose_secret(),
            test_db_name
        );

        let connection_pool = PgPool::connect(&test_db_connection_string)
            .await
            .expect("Failed to connect to Postgres.");
        sqlx::migrate!("./migrations")
            .run(&connection_pool)
            .await
            .expect("Failed to migrate the database");

        connection_pool
    }
}

// default starttls => set as config ?
#[derive(Deserialize)]
pub struct EmailConfig {
    pub sender: String,
    pub user: String,
    pub password: Secret<String>,
    pub host: String,
}

pub fn parse_email_config() -> EmailConfig {
    dotenv().ok();
    envy::prefixed("EMAIL_")
        .from_env::<EmailConfig>()
        .expect("Missing email config environment variables")
}

impl EmailConfig {
    pub fn email_client(&self) -> EmailClient {
        let creds = Credentials::new(
            self.user.to_owned(),
            self.password.expose_secret().to_owned(),
        );
        let mailer = AsyncSmtpTransport::<Tokio1Executor>::starttls_relay(&self.host)
            .unwrap()
            .credentials(creds)
            .build();
        EmailClient {
            sender: self.sender.parse().unwrap(),
            mailer,
        }
    }
}
