use crate::domain::NewSubscriber;
use actix_web::{post, web, HttpResponse, Responder};
use chrono::Utc;
use serde::Deserialize;
use sqlx::{Error, PgPool};
use uuid::Uuid;

#[derive(Debug, Deserialize)]
pub struct FormData {
    pub email: String,
    pub name: String,
}

fn query_error_is_duplicate_key(error: Error) -> bool {
    match error {
        Error::Database(db_error) => match db_error.code() {
            Some(code) => code == "23505",
            _ => false,
        },
        _ => false,
    }
}

#[tracing::instrument(
    name = "Saving new subscriber details in the database",
    skip(new_subscriber, pool)
)]
pub async fn insert_subscriber(
    pool: &PgPool,
    new_subscriber: &NewSubscriber,
) -> Result<(), sqlx::Error> {
    match sqlx::query!(
        r#"
		INSERT INTO subscriptions (id, email, name, subscribed_at) VALUES ($1, $2, $3, $4)
		"#,
        Uuid::new_v4(),
        new_subscriber.email.as_ref(),
        new_subscriber.name.as_ref(),
        Utc::now()
    )
    .execute(pool)
    .await
    {
        Err(e) => {
            tracing::error!("Failed to execute query: {:?}", e);
            Err(e)
        }
        _ => Ok(()),
    }
}

#[post("/subscriptions")]
#[tracing::instrument(
    name = "Adding a new subscriber",
    skip(form, pool),
    fields(
        subscriber_email = %form.email,
        subscriber_name = %form.name
    )
)]
pub async fn subscriptions(form: web::Form<FormData>, pool: web::Data<PgPool>) -> impl Responder {
    let new_subscriber = match NewSubscriber::try_from(form.0) {
        Ok(value) => value,
        Err(message) => {
            return HttpResponse::BadRequest().body(format!("form data error: {}", message))
        }
    };
    match insert_subscriber(&pool, &new_subscriber).await {
        Ok(_) => HttpResponse::Created().finish(),
        Err(e) => match query_error_is_duplicate_key(e) {
            true => HttpResponse::Conflict().finish(),
            false => HttpResponse::InternalServerError().finish(),
        },
    }
}
